/*
** iplog_lockfile.c - iplog lockfile management.
** Copyright (C) 1999 behe <eric@ojnk.net>, Odin <odin@numb.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
*/

#include <config.h>

#define _GNU_SOURCE
#include <unistd.h>
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <sys/stat.h>
#include <sys/file.h>

#include <iplog.h>

#ifndef _PATH_VARRUN
#	define LOCKFILE "/var/run/iplog.pid"
#else
#	define LOCKFILE (_PATH_VARRUN "iplog.pid")
#endif

void write_lockfile(void) {
	u_char buf[16];
	ssize_t len;
	int fd;
	u_int pid;
	struct stat st;

	fd = open(LOCKFILE, O_RDWR | O_CREAT | O_TRUNC | O_EXCL, 0644);

	if (fd == -1) {
		if (errno != EEXIST)
			fatal("Cannot open lockfile \"%s\": %s", LOCKFILE, strerror(errno));
		fd = open(LOCKFILE, O_RDONLY);
		if (fd == -1)
			fatal("Cannot open lockfile for reading");
		len = read(fd, buf, sizeof(buf));
		if (len < 1)
			fatal("Cannot read lockfile \"%s\": %s", LOCKFILE, strerror(errno));
		buf[len] = '\0';
		pid = strtoul(buf, NULL, 10);

		if (kill(pid, 0) && errno == ESRCH) {
			if (unlink(LOCKFILE) == -1)
				fatal("Could not remove stale lockfile \"%s\"", LOCKFILE);
		} else if (flock(fd, LOCK_EX | LOCK_NB) == -1)
			fatal("iplog is already running (pid: %d)", pid);

		close(fd);
		fd = open(LOCKFILE, O_RDWR | O_CREAT | O_TRUNC, 0644);
		if (fd == -1)
			fatal("Cannot open lockfile \"%s\": %s", LOCKFILE, strerror(errno));
	}

	if (fstat(fd, &st) == -1)
		fatal("Cannot stat lockfile \"%s\": %s", LOCKFILE, strerror(errno));

	if (!S_ISREG(st.st_mode))
		fatal("Lockfile \"%s\" is not a regular file.", LOCKFILE);

	dprintf(fd, "%d\n", getpid());

	if (flock(fd, LOCK_EX | LOCK_NB) == -1)
		fatal("Lockfile \"%s\" could not be locked: %s", LOCKFILE, strerror(errno));
}

void kill_iplog(int sig) {
	struct stat st;
	int fd;
	u_char buf[16];
	ssize_t len;
	pid_t pid;

	if (lstat(LOCKFILE, &st) == -1)
		fatal("iplog is not running.");
	
	fd = open(LOCKFILE, O_RDONLY);
	if (fd == -1)
		fatal("Can't open pid file: %s", strerror(errno));

	len = read(fd, buf, sizeof(buf));
	if (len < 1)
		fatal("Error reading pid.");

	buf[len] = '\0';
	pid = strtoul(buf, NULL, 10);

	if (kill(pid, sig) == -1) {
		unlink(LOCKFILE);
		fatal("Can't send process signal %d: %s", sig, strerror(errno));
	}

	if (sig != 1 && unlink(LOCKFILE) == -1)
		fatal("Can't unlink lockfile: %s", strerror(errno));

	exit(0);
}

/* vim:ts=4:sw=8:tw=0 */
